using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

namespace Campus
{
    class Scholarship : Structure
    {
        static public Texture2D BuildingTexture { get; private set; }
        static public Texture2D StaticTeamColorTexture { get; private set; }
        static public Texture2D ConstructionTexture { get; private set; }
        static public Vector2 StaticOrigin { get; private set; }
        static public List<Vector2> StaticTilesOccupied { get; private set; }
        static public void InitializeTextures(ContentManager Content) {
            BuildingTexture = Content.Load<Texture2D>("Structures/building_scholarship_base");
            StaticTeamColorTexture = Content.Load<Texture2D>("Structures/building_scholarship_team");
            ConstructionTexture = Content.Load<Texture2D>("Structures/building_scholarship_build");
            StaticOrigin = new Vector2(68, 195);

            StaticTilesOccupied = new List<Vector2>();
            StaticTilesOccupied.Add(new Vector2(0, 0));  // Tile this building is placed on.
            StaticTilesOccupied.Add(new Vector2(0, 1));
            StaticTilesOccupied.Add(new Vector2(0, 2));
            StaticTilesOccupied.Add(new Vector2(1, 0));
            StaticTilesOccupied.Add(new Vector2(1, 1));
            StaticTilesOccupied.Add(new Vector2(1, 2));
            StaticTilesOccupied.Add(new Vector2(2, 0));
            StaticTilesOccupied.Add(new Vector2(2, 1));
            StaticTilesOccupied.Add(new Vector2(2, 2));
        }

        private void CreateAbilities() {
            Abilities.Add("Garrison", new GarrisonAbility(MenuIcons.MoveIcon, this, 100, delegate(Unit u) { return true; }));
            Abilities.Add("Expell Units", new ExpellUnitsAbility(MenuIcons.AttackIcon, this));
        }

        public Scholarship(Player owner, Level level)
            : base(BuildingTexture, StaticTeamColorTexture, ConstructionTexture, StaticTilesOccupied, StaticOrigin, Vector2.Zero, 100, owner, level, new Dictionary<string, Ability>(), new Dictionary<string, AbilitySubMenu>())
        {
            m_name = "Scholarship";
            CreateAbilities();

            //TODO: Create abilities

        }
        public override string ToString()
        {
            string returnString = m_name + "\n" +
                                  "HP: " + m_currentHP + "/" + m_maxHP + "\nThis is your money gaining facility \nput up to 4 workers in here";
            return returnString;
        }
        public override void Update(Player player)
        {
            player.AddMoney(((GarrisonAbility)Abilities["Garrison"]).GarrisonedUnits.Count * 4);
        }
    }
}